package engine_lsd;

/**
 * Implements a priority queue via a binary heap.
 * 
 * @author Mark Allen Weiss modified by Francesco Romani
 */
public class PriorityQueue {
	private static final int DEFAULT_CAPACITY = 100;

	private int currentSize; // Number of elements in heap

	private State[] array; // The heap array

	/**
	 * Construct the priority queue
	 */
	public PriorityQueue() {
		this.currentSize = 0;
		this.array = new State[DEFAULT_CAPACITY + 1];
	}

	/**
	 * Insert into the priority queue.
	 * 
	 * @param x
	 *            the item to insert.
	 */
	public void put(State x) {
		if (currentSize + 1 == array.length)
			doubleArray();
		// Percolate up
		int hole = ++currentSize;
		for (; hole > 1 && x.value() < array[hole / 2].value(); hole /= 2)
			array[hole] = array[hole / 2];
		array[hole] = x;
	}

	/**
	 * Find the smallest item in the priority queue.
	 * 
	 * @return the smallest item, null if empty
	 */
	public State look() {
		if (isEmpty())
			return null;
		return array[1];
	}

	/**
	 * Remove the smallest item from the priority queue.
	 * 
	 * @return the smallest item, null if empty
	 */
	public State get() {
		if (isEmpty())
			return null;
		State minItem = look();
		array[1] = array[currentSize--];
		percolateDown(1);
		return minItem;
	}

	/**
	 * Test if the priority queue is logically empty.
	 * 
	 * @return true if empty, false otherwise.
	 */
	public boolean isEmpty() {
		return currentSize == 0;
	}

	/**
	 * Returns size.
	 * 
	 * @return current size.
	 */
	public int size() {
		return currentSize;
	}

	/**
	 * Make the priority queue logically empty.
	 */
	public void clear() {
		currentSize = 0;
		array = new State[DEFAULT_CAPACITY + 1];
	}

	/**
	 * Internal method to percolate down in the heap.
	 * 
	 * @param hole
	 *            the index at which the percolate begins.
	 */
	private void percolateDown(int hole) {
		int child;
		State tmp = array[hole];

		for (; hole * 2 <= currentSize; hole = child) {
			child = hole * 2;
			if (child != currentSize
					&& array[child + 1].value() < array[child].value())
				child++;
			if (array[child].value() < tmp.value())
				array[hole] = array[child];
			else
				break;
		}
		array[hole] = tmp;
	}

	/**
	 * Internal method to extend array.
	 */
	private void doubleArray() {
		State[] newArray = new State[array.length * 2];
		System.arraycopy(array, 0, newArray, 0, array.length);
		array = newArray;
	}
}