package sudoku_human;

import java.util.PriorityQueue;

import elab_stats.Statistiche;


/**
 * Classe che definisce una griglia composta da 9x9 caselle,
 * utilizzabile durante l'esecuzione dell'algoritmo di risoluzione
 * del sudoku per rappresentare uno stato.<br>
 * 
 * one line to give the program's name and an idea of what it does.
 * Copyright (C) 2005  Marco Cornolti (cornolti@cli.di.unipi.it)<br>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.<br>
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.<br>
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.<br>
 * 
 * @author Marco Cornolti
 * 
 */

public class Griglia implements Comparable{
    private boolean verificata=false;
    private Casella[][] griglia;
	private int caselleDefinite;
	
	/**Da un array di 9x9 caselle, costruisce uno stato.
	 * @param contenuto le caselle da inserire nella griglia
	 */
	public Griglia (Casella[][] contenuto){
		griglia= contenuto;
		for (int x=0;x<9;x++)
			for (int y=0;y<9;y++){
				if (griglia[x][y].getValue()!=0) caselleDefinite++;
			}
	}
	
	
	/**Da un array di 9x9 interi rappresentanti il valore delle caselle,
	 * costruisce uno stato. Se una casella vale 0, la casella diventa indefinita.
	 * @param contenuto array di 9x9 interi con le caselle definite.
	 */
	public Griglia (int[][]contenuto){
		griglia= new Casella[9][9];
		caselleDefinite=0;
		for (int x=0;x<9;x++)
			for (int y=0;y<9;y++){
				griglia [x][y]=new Casella(contenuto[y][x]);
				if (contenuto[y][x]!=0) caselleDefinite++;
				
			}
	}
	
	/** lancia l'algoritmo per risolvere la griglia.
	 * @return null se e' impossibile trovare la soluzione, altrimenti la griglia 
	 * rappresentante la soluzione.
	 */
	public Griglia risolvi(){
	    PriorityQueue statesQueue = new PriorityQueue();
	    statesQueue.add(this);
	    while (!statesQueue.isEmpty()){
	        //System.out.println("Nmero elementi in coda:"+statesQueue.size());
	        Griglia currentGriglia = (Griglia) statesQueue.poll();
	        if (currentGriglia.risolviIstanza(statesQueue)&& currentGriglia.verifica()){
	            return currentGriglia;
	        }
	    }
	    return null;

	}
	
	/** esegue un passaggio per avvicinarsi alla risoluzione di una singola istanza.
	 * se i metodi di scan normali non hanno avuto effetto ma la griglia non e' ancora
	 * giunta ad una contraddizione, inserisce nella coda le istanze semplificate da
	 * provare a risolvere. 
	 * @return true ses e' riuscito a cambiare qualcosa
	 */
	private boolean risolviIstanza(PriorityQueue statesQueue){
	    boolean cambiata=false;
	    while (this.caselleDefinite()<81){
	        cambiata=false;
	        
            cambiata=cambiata|this.scanARigaColonna();
            cambiata=cambiata|this.scanBRigaColonna();
            cambiata=cambiata|this.scanABBlocco();
            if (!cambiata){
                cambiata=cambiata|this.scanC();
            }
	        //System.out.println("Caselle definite: " +this.caselleDefinite());
	        
	        if (!this.possibile()) {
                //System.out.println("la griglia e' giunta ad una contraddizione.");
	            return false;
            }
            
	        if (!cambiata) {
	        	float combinazioni=1;
	        	for (int x=0; x<9;x++)
	        		for (int y=0; y<9; y++)
	        			combinazioni*=this.griglia[x][y].getPossibleValuesCount();

	            int[] ordineControllo = sortByPossibilityCountAndImpact();
	                
	            //0 e' l'elemento con priorita' ma non e' detto che abbia possibilita'>1
	            int[] possibilita = this.griglia[0][0].getPossibleValues();
	            int x=0,y=0;
	            for (int casella=0;casella<81;casella++) {
		            
	            y = ordineControllo[casella]%9;
	            x = ordineControllo[casella]/9;
	            
	            possibilita = this.griglia[x][y].getPossibleValues();
	            if (possibilita.length>1) break;
	            }
	            if (possibilita.length>1){
	                    //ordiniamo il vettore possibilita', in modo che venga ordinata per prima la possibilita'
	                    //la cui espansione e' piu' piccola. ovvero quella che elimina piu' corrispondenze nella
	                    //tabella, ovvero quella con piu' corrispondenze tra le possibilita' nella tabella.
	                    sortByOccurrences(possibilita,x,y);
	                
	                    for (int possDaProvare=0;possDaProvare<possibilita.length;possDaProvare++) {
	                        //for (int possDaProvare=possibilita.length-1;possDaProvare>=0;possDaProvare--) {
	                        
	                        Griglia grigliaDaProvare = this.cloneModificato(x,y,possibilita[possDaProvare]);
	                        statesQueue.add(grigliaDaProvare);
	                        Statistiche.aumentaEspansioni();
	                    }
	                }
	            return false;
	        }
	    }
	    
	    
	    return cambiata;
	    
	    
	}
	
		
	public int caselleDefinite(){
		return this.caselleDefinite;
	}
	
	/**
     * OBSOLETO: e' piu' potente scanA
	 * scan di ogni casella:
	 * D) se viste le altre caselle della riga, della colonna e del blocco, la casella e'
	 * l'unica che puo' essere in un modo, verra' definita in quel modo.
     * funzionamento:
     * -definizione delle possibilita' nella riga
     * -definizione delle possibilita' nella colonna
     * -definizione delle possibilita' nel blocco
     * -intersezione dei 3 insiemi
     * -se intersezione ha cadinalita' 8, la possibilita' esclusa va assegnata alla casella.
	 * @return true ses e' stata applicata una qualsiasi modifica
	 */
	private boolean scanD(){
	    boolean cambiata=false;
	    for (int x=0;x<9;x++)
	        for (int y=0;y<9;y++){
	            
	            if (griglia[x][y].isUndefined()){
                                        
	                //scan riga
                    boolean[] possRiga = new boolean[9];
                    for (int i=0;i<9;i++) possRiga[i]=false;
                    for (int xDaControllare=0;xDaControllare<9;xDaControllare++){
	                    if (xDaControllare==x) continue;
                        int[] possDaAggiungere = griglia[xDaControllare][y].getPossibleValues();
	                    for (int i=0;i<possDaAggiungere.length;i++) 
	                        possRiga[possDaAggiungere[i]-1]=true;
	                }
                    
                    //scan colonna
                    boolean[] possCol = new boolean[9];
                    for (int i=0;i<9;i++) possCol[i]=false;
                    for (int yDaControllare=0;yDaControllare<9;yDaControllare++){
	                    if (yDaControllare==y) continue;
	                    int[] possDaAggiungere = griglia[x][yDaControllare].getPossibleValues();
	                    for (int i=0;i<possDaAggiungere.length;i++) 
	                        possCol[possDaAggiungere[i]-1]=true;
	                }

                    //scan blocco
                    boolean[] possBloc = new boolean[9];
                    for (int i=0;i<9;i++) possBloc[i]=false;
                    for (int yDaControllare=(y/3)*3;yDaControllare<(y/3)*3+3;yDaControllare++)
	                    for (int xDaControllare=(x/3)*3;xDaControllare<(x/3)*3+3;xDaControllare++){
	                        if (yDaControllare==y||xDaControllare==x) continue;
	                        int[] possDaAggiungere = griglia[xDaControllare][yDaControllare].getPossibleValues();
	                        for (int i=0;i<possDaAggiungere.length;i++) 
	                            possBloc[possDaAggiungere[i]-1]=true;
	                    }
                    
                    int possCount=0;
                    for(int i=0;i<9;i++) if(possRiga[i] || possCol[i] || possBloc[i]) possCount++;
                    
                    if (possCount==8){
                        int rightVal=0;
                        for(int i=0;i<9;i++){
                            if (!(possRiga[i] || possCol[i] || possBloc[i])) {
                                rightVal=i+1;
                                break;
                            }
                        }
                        this.griglia[x][y].afferma(rightVal);
                    }
                    else if (possCount==0){
                        System.out.println("ERRORE!");
                        System.exit(0);
                    }
                    
	                
	                    
	            if (!griglia[x][y].isUndefined()) {
                    cambiata=true;
	                caselleDefinite++;
	                //	System.out.println("2 DEFINITE:"+caselleDefinite+" X:"+x+" Y:"+y+" VALORE:"+griglia[x][y].getValue());
	            }
                }
	        }	
	    
	    
	    return cambiata;
	}
    
    
    /**
     * scan di ogni casella:
     * A) se viste le altre caselle della riga e della la colonna, la casella e'
     * l'unica che puo' essere in un modo, verra' definita in quel modo.
     * @return true ses e' stata applicata una qualsiasi modifica
     */
    private boolean scanARigaColonna(){
        boolean cambiata=false;
        for (int x=0;x<9;x++)
            for (int y=0;y<9;y++){
                
                
                int[] possibilita = griglia[x][y].getPossibleValues();
                
                if (possibilita.length>1){
                    //A) per colonna
                    for (int numPossDaControllare=0;numPossDaControllare<possibilita.length;numPossDaControllare++){
                        
                        boolean altraPresenza=false;
                        int yDaControllare=0;
                        while (!altraPresenza&&yDaControllare<9) { //non ci possono essere altre caselle nella riga, oltre quella corrente, con la stessa possibilita'.
                            if (yDaControllare!=y && griglia[x][yDaControllare].getPossibilita(possibilita[numPossDaControllare])) altraPresenza=true;
                            yDaControllare++;
                        }
                        if (!altraPresenza) {
                            griglia[x][y].afferma(possibilita[numPossDaControllare]);
                            cambiata=true;
                            //  System.out.println("1 DEFINITE:"+caselleDefinite+" X:"+x+" Y:"+y+" VALORE:"+griglia[x][y].getValue());
                            break;
                        }
                    }
                    
                    
                    possibilita = griglia[x][y].getPossibleValues();
                    if (possibilita.length>1)
                        //A) per riga
                        for (int numPossDaControllare=0;numPossDaControllare<possibilita.length;numPossDaControllare++){
                            
                            boolean altraPresenza = false;
                            int xDaControllare=0;
                            while (!altraPresenza && xDaControllare<9) {
                                if (xDaControllare!=x && griglia[xDaControllare][y].getPossibilita(possibilita[numPossDaControllare])) altraPresenza=true;
                                xDaControllare++;
                            }
                            if (!altraPresenza) {
                                griglia[x][y].afferma(possibilita[numPossDaControllare]);
                                cambiata=true;
                                //  System.out.println("1 DEFINITE:"+caselleDefinite+" X:"+x+" Y:"+y+" VALORE:"+griglia[x][y].getValue());
                                break;
                            }
                        }
                    
                    if (!griglia[x][y].isUndefined()) {
                        caselleDefinite++;
                        //  System.out.println("2 DEFINITE:"+caselleDefinite+" X:"+x+" Y:"+y+" VALORE:"+griglia[x][y].getValue());
                    }
                }
            }   
        
        
        return cambiata;
    }
    
    /** 
     * scan di ogni casella:
     * B) se in una riga ci sono caselle definite, tutte le altre caselle della stessa riga e colonna
     * non possono avere la possibilita' della casella definita.
     */
     private boolean scanBRigaColonna(){
	    
	    boolean cambiata=false;
	    for (int x=0;x<9;x++)
	        for (int y=0;y<9;y++){
	            if (griglia[x][y].isUndefined()){ 
	                
	                //B) per colonna
	                for (int yDaControllare=0;yDaControllare<9;yDaControllare++)
	                    if (griglia[x][y].getValue()==0)
	                        cambiata = cambiata | griglia[x][y].togliPossibilita(griglia[x][yDaControllare].getValue());
	                
	                //B) per riga
	                for (int xDaControllare=0;xDaControllare<9;xDaControllare++)
	                    if (griglia[x][y].getValue()==0)
	                        cambiata = cambiata | griglia[x][y].togliPossibilita(griglia[xDaControllare][y].getValue());
	                
	                if (!griglia[x][y].isUndefined()) {
	                    caselleDefinite++;
	                    //  System.out.println("2 DEFINITE:"+caselleDefinite+" X:"+x+" Y:"+y+" VALORE:"+griglia[x][y].getValue());
	                }
	            }
	        }
	    return cambiata;
    }
	
     /** Applicazione delle regole A e B ai blocchi.
     * A) se viste le altre caselle del blocco, la casella e'
     * l'unica che puo' essere in un modo, verra' definita in quel modo.
     * B) se in un blocco ci sono caselle definite, tutte le altre caselle dello stesso blocco
     * non possono avere la possibilita' della casella definita.
     * @return true ses la griglia e' cambiata.
     */
    private boolean scanABBlocco(){
        boolean cambiata=false;
        for (int xBlocco=0;xBlocco<3;xBlocco++)
            for (int yBlocco=0;yBlocco<3;yBlocco++){
                
                int[] possibilita;
                for (int x=0;x<3;x++)
                    for (int y=0;y<3;y++){
                        
                        possibilita= griglia[x+3*xBlocco][y+3*yBlocco].getPossibleValues();
                        if (possibilita.length>1){
                            // A) per bloccoSystem.out.println()
                            for (int numPossDaControllare=0;numPossDaControllare<possibilita.length;numPossDaControllare++){
                                
                                boolean altraPossibilita=false;
                                int xDaControllare=0;
                                int yDaControllare=0;
                                while (!altraPossibilita&&xDaControllare<3) {
                                    while (!altraPossibilita&&yDaControllare<3) {
                                        if ( !(xDaControllare==x && yDaControllare ==y) &&
                                                griglia[3*xBlocco+xDaControllare][3*yBlocco+yDaControllare].getPossibilita(possibilita[numPossDaControllare])) altraPossibilita= true;
                                        yDaControllare++;
                                        }
                                    xDaControllare++;
                                    yDaControllare=0;
                                }
                                if (!altraPossibilita) {
                                    griglia[x+3*xBlocco][y+3*yBlocco].afferma(possibilita[numPossDaControllare]);
                                    cambiata=true;
                                    //  System.out.println("1 DEFINITE:"+caselleDefinite+" X:"+x+" Y:"+y+" VALORE:"+griglia[x][y].getValue());
                                    break;}
                            }
                            
                            
                            //B) per blocco
                            for (int yDaControllare=0;yDaControllare<3;yDaControllare++)
                                for (int xDaControllare=0;xDaControllare<3;xDaControllare++)
                                    if (griglia[x+3*xBlocco][y+3*yBlocco].getValue()==0)
                                        cambiata = cambiata | griglia[x+3*xBlocco][y+3*yBlocco].togliPossibilita(griglia[xDaControllare+3*xBlocco][yDaControllare+3*yBlocco].getValue());
                                    
                                    
                                    
                            if (!griglia[x+3*xBlocco][y+3*yBlocco].isUndefined()) 
                                caselleDefinite++;
                            //                  System.out.println("4 DEFINITE:"+caselleDefinite+" X:"+(x+3*xBlocco)+" Y:"+(y+3*yBlocco)+" VALORE:"+griglia[x+3*xBlocco][y+3*yBlocco].getValue());
                            
                        }
                        
                    }
            }
        return cambiata;
    }
    
    /**Applicazione della regola C.
     * @return true ses e' stata eliminata almeno una possibilita'
     */
    private boolean scanC(){
    	boolean cambiata=false;
    	
    	for (int xBlocco=0;xBlocco<3;xBlocco++)
    		for (int yBlocco=0;yBlocco<3;yBlocco++){
    			
    			for (int x=0;x<3;x++)
    				for (int y=0;y<3;y++){
    					
    					int[] possibilita= griglia[x+3*xBlocco][y+3*yBlocco].getPossibleValues();
    					if (possibilita.length>1)
    						//se tutte le possibilita' di un numero sono sulla stessa riga...
    						for (int numPossDaControllare=0;numPossDaControllare<possibilita.length;numPossDaControllare++){
    							boolean indefinizioneFuoriRiga=false;
    							
    							int xDaControllare=0;
    							int yDaControllare=0;
    							while (yDaControllare<3 && !indefinizioneFuoriRiga){
    								while (xDaControllare<3&&!indefinizioneFuoriRiga){
    									if (yDaControllare!=y && griglia[xDaControllare+3*xBlocco][yDaControllare+3*yBlocco].getPossibilita(possibilita[numPossDaControllare])) indefinizioneFuoriRiga=true;
    									xDaControllare++;
    								}
    								yDaControllare++;
    								xDaControllare=0;
    							}
    							
    							if (!indefinizioneFuoriRiga) {
    								for (int xDaCambiare=0;xDaCambiare<9;xDaCambiare++){
    									if (((int)xDaCambiare/3)!=xBlocco) {
    										int indefinizionePrima = griglia[xDaCambiare][y+3*yBlocco].getPossibleValues().length;
    										cambiata = cambiata|griglia[xDaCambiare][y+3*yBlocco].togliPossibilita(possibilita[numPossDaControllare]);
    										if (griglia[xDaCambiare][y+3*yBlocco].getPossibleValues().length==1 && indefinizionePrima!=1) caselleDefinite++;
    									}
    									
    								}
    							}
    						}
    					
    					
    					//se tutte le possibilita' di un numero sono sulla stessa colonna...
    					for (int numPossDaControllare=0;numPossDaControllare<possibilita.length;numPossDaControllare++){
    						boolean indefinizioneFuoriColonna=false;
    						
    						int xDaControllare=0;
    						int yDaControllare=0;
    						while (yDaControllare<3 && !indefinizioneFuoriColonna){
    							while (xDaControllare<3&&!indefinizioneFuoriColonna){
    								if (xDaControllare!=x && griglia[xDaControllare+3*xBlocco][yDaControllare+3*yBlocco].getPossibilita(possibilita[numPossDaControllare])) indefinizioneFuoriColonna=true;
    								xDaControllare++;
    							}
    							yDaControllare++;
    							xDaControllare=0;
    						}
    						
    						if (!indefinizioneFuoriColonna) {
    							for (int yDaCambiare=0;yDaCambiare<9;yDaCambiare++){
    								if (((int)yDaCambiare/3)!=yBlocco) {
    									int indefinizionePrima = griglia[x+3*xBlocco][yDaCambiare].getPossibleValues().length;
    									
    									cambiata = cambiata|griglia[x+3*xBlocco][yDaCambiare].togliPossibilita(possibilita[numPossDaControllare]);
    									if (griglia[x+3*xBlocco][yDaCambiare].getPossibleValues().length==1 && indefinizionePrima!=1) caselleDefinite++;
    								}		
    							}
    						}
    					}
    				}	
    		}
    	return cambiata;
    }
    
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		String result="";
		for (int x=0;x<9;x++){
			for (int y=0;y<9;y++){
				int[]possibilita=griglia[y][x].getPossibleValues();
				for (int i=0;i<possibilita.length;i++) result+=possibilita[i];
				result+="  ";
			}
			result+="\n";
		}
	return result;
	}
	
	/**
	 * @return la stringa rappresentante la griglia con 0 al posto delle caselle non
	 * definite
	 */
	public String toStringUndefined(){
		String result="";
		for (int x=0;x<9;x++){
			for (int y=0;y<9;y++){
				result+=griglia[y][x].getValue();
				result+="  ";
			}
			result+="\n";
		}
	return result;
	}

	/** verifica che un'istanza sia una soluzione, ovvero che siano applicate tutte le 
	 * regole del sudoku
	 * @return true ses l'istanza rispetta tutte le regole del sudoku.
	 */
	public boolean verifica() {
	    if (this.verificata) return true;
		if (this.caselleDefinite()<81) return false;
		
		//scan del blocco
		for (int xBlocco=0;xBlocco<3;xBlocco++)
			for (int yBlocco=0;yBlocco<3;yBlocco++){
				boolean[] scelte = new boolean[9];
				for (int x=0;x<3;x++)
					for (int y=0;y<3;y++){
						if (griglia[x+3*xBlocco][y+3*yBlocco].getValue()==0) return false; 
						scelte[griglia[x+3*xBlocco][y+3*yBlocco].getValue()-1]=true;
					}
				for (int i=0;i<9;i++) if (scelte[i]==false) return false;
				
			}
		
		//scan delle colonne
		for (int x=0;x<9;x++) {
			boolean[] scelte = new boolean[9];
			for (int y=0;y<9;y++){
				if (griglia[x][y].getValue()==0) return false; 
				scelte[griglia[x][y].getValue()-1]=true;
			}
			for (int i=0;i<9;i++) if (scelte[i]==false) return false;
		}
		
		//scan delle colonne
		for (int y=0;y<9;y++) {
			boolean[] scelte = new boolean[9];
			for (int x=0;x<9;x++){
				if (griglia[x][y].getValue()==0) return false; 
				scelte[griglia[x][y].getValue()-1]=true;
			}
			for (int i=0;i<9;i++) if (scelte[i]==false) return false;
		}
		
		this.verificata=true;
		return true;
	}
	/**
	 * @return true ses le caselle attualmente determinate non sono in contraddizione tra loro.
	 */
	public boolean possibile() {
		
		//scan del blocco
		for (int xBlocco=0;xBlocco<3;xBlocco++)
			for (int yBlocco=0;yBlocco<3;yBlocco++){
				boolean[] scelte = new boolean[9];
				for (int x=0;x<3;x++)
					for (int y=0;y<3;y++){
					    int valore = griglia[x+3*xBlocco][y+3*yBlocco].getValue();
						if (valore!=0){
						    if (scelte[valore-1]) return false;
						    scelte[valore-1]= true;
						}
					}
				
			}
		
		//scan delle colonne
		for (int x=0;x<9;x++) {
			boolean[] scelte = new boolean[9];
			for (int y=0;y<9;y++){
			    int valore = griglia[x][y].getValue();
				if (valore!=0) {
				    if (scelte[valore-1]) return false;
				    scelte[valore-1]=true;
				}
				
			}
		}
		
		//scan delle colonne
		for (int y=0;y<9;y++) {
			boolean[] scelte = new boolean[9];
			for (int x=0;x<9;x++){
			    int valore = griglia[x][y].getValue();
				if (valore!=0){
				    if (scelte[valore-1]) return false;
				    scelte[valore-1]=true;
				}
			}
		}
		
		
		return true;
	}
	
	
	/** restituisce un clone della griglia con l'eccezione di una casella definita diversamente
	 * @param xDaModificare coordinata x della casella da definire
	 * @param yDaModificare coordinata y della casella da definire
	 * @param n intero col quale definire la casella.
	 * @return la griglia modificata
	 */
	public Griglia cloneModificato(int xDaModificare, int yDaModificare, int n){
		Casella[][] nuoveCaselle = new Casella[9][9];
		for (int x=0;x<9;x++) 
			for (int y=0;y<9;y++) {
				nuoveCaselle[x][y]= new Casella(this.griglia[x][y].getPossibleValues());
			}
		nuoveCaselle[xDaModificare][yDaModificare].afferma(n);
	
		return new Griglia(nuoveCaselle);
		
	}
	
	/**
	 * @param n il numero da 1 a 9
	 * @return la quantita' di caselle che ammettono quel numero come possibilita' nel blocco, nella riga e nella colonna
	 */
	public int countOccurrences(int n, int xcasella, int ycasella){
	    int count=1;
	    int xblocco=xcasella/3;
	    int yblocco=ycasella/3;
	    for (int x=0;x<9;x++) if (x/3!=xblocco && x!=xcasella && this.griglia[x][ycasella].getPossibilita(n)) count++;
	    for (int y=0;y<9;y++) if (y/3!=yblocco && y!=ycasella && this.griglia[xcasella][y].getPossibilita(n)) count++;
	    for (int x=0;x<3;x++)
	        for (int y=0;y<3;y++)
	            if (3*xblocco+x != xcasella && 3*yblocco+y!=ycasella
	                    && this.griglia[3*xblocco+x][3*yblocco+y].getPossibilita(n)) count++;
	    
	    return count;
	}
	
	
	/**ordina un array con le possibilita' per una casella. e' messa per prima la possibilita' che ha piu' occorrenze
	 * tra le possibilita' del resto del blocco, della riga, e della linea.
	 * @param possibilita array con le possibilita' 1-9 da ordinare
	 */
	public void sortByOccurrences(int[] possibilita, int x, int y){
	    int[] occorrenze = new int[possibilita.length];
        for (int i=0;i<possibilita.length;i++)
            occorrenze[i]=this.countOccurrences(possibilita[i],x,y);
            
        //ordina le possibilita' in maniera decrescente servendosi dell'array ausiliario occorrenze
        Utils.sortByWeightDecrease(occorrenze,possibilita);
	}
	/**Ordinamento delle caselle dando priorita' a quelle con meno possibilita',
	 * diamo priorita' anche a quelle con maggiore impatto tra le varie possibilita', ovvero per una singola
	 * possibilita' hanno maggiore possibilita'.         
	 * @return l'array con le posizioni delle caselle. array[i] dove i%9 e' la coordinata y e i/9 la coordinata x
	 */
	public int[] sortByPossibilityCountAndImpact(){
	    int[] tempPeso = new int[81];
	    int[] tempCaselle = new int[81];
	    for (int i=0;i<tempCaselle.length;i++) tempCaselle[i]=i;
	    
	    for (int x=0;x<9;x++)
	        for (int y=0;y<9;y++){
	            int[] possibilita = this.griglia[x][y].getPossibleValues();
	            tempPeso[9*x+y]=(-possibilita.length)*10;
	            int maxImpact=0;
	            //ha senso cercare possibilita' ses le possibilita' sono !=1, altrimenti saranno tutte zero
	            for (int i=0;possibilita.length!=1 && i<possibilita.length;i++){
	                int impact = this.countOccurrences(possibilita[i],x,y); 
	                if (maxImpact<impact) maxImpact=impact;
	            }
	            tempPeso[9*x+y]+=maxImpact;
	        }
	    
	    Utils.sortByWeightDecrease(tempPeso,tempCaselle);
	    return tempCaselle;
	}


	public int compareTo(Object arg0) {
		return 0;
	}
	
	
}
